/*
 *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "galeon-window.h"
#include "galeon-sidebar.h"
#include "gul-state.h"
#include "statusbar.h"
#include "toolbar.h"
#include "ppview-toolbar.h"
#include "window-commands.h"
#include "find-dialog.h"
#include "history-dialog.h"
#include "popup-commands.h"
#include "galeon-shell.h"
#include "bookmarks-bonoboui-menu.h"
#include "bookmarks-bonoboui-tb.h"
#include "gul-bonobo-extensions.h"
#include "eel-gconf-extensions.h"
#include "prefs-strings.h"
#include "galeon-embed-utils.h"
#include "bookmarks-editor-dockable.h"
#include "window-recent-history-menu.h"

#include <string.h>
#include <bonobo/bonobo-window.h>
#include <bonobo/bonobo-i18n.h>
#include <bonobo/bonobo-ui-util.h>
#include <bonobo/bonobo-ui-component.h>
#include <libgnomevfs/gnome-vfs-uri.h>
#include <gtk/gtk.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <gdk/gdkx.h>
#include <gdk/gdkkeysyms.h>

#define CHARSET_MENU_PATH "/menu/View/EncodingMenuPlaceholder"
#define GO_RECENT_HISTORY_PATH "/menu/Go/RecentHistory"

#define GO_BACK_CMD_PATH "/commands/GoBack"
#define GO_FORWARD_CMD_PATH "/commands/GoForward"
#define GO_UP_CMD_PATH "/commands/GoUp"
#define EDIT_FIND_NEXT_CMD_PATH "/commands/EditFindNext"
#define EDIT_FIND_PREV_CMD_PATH "/commands/EditFindPrev"
#define VIEW_SIDEBAR_PATH "/commands/View Sidebar"
#define VIEW_MENUBAR_PATH "/commands/View Menubar"
#define VIEW_STATUSBAR_PATH "/commands/View Statusbar"
#define VIEW_TOOLBAR_PATH "/commands/View Toolbar"
#define VIEW_BOOKMARKSBAR_PATH "/commands/View BookmarksBar"
#define VIEW_FULLSCREEN_PATH "/commands/View Fullscreen"

#define ID_VIEW_SIDEBAR "View Sidebar"
#define ID_VIEW_MENUBAR "View Menubar"
#define ID_VIEW_STATUSBAR "View Statusbar"
#define ID_VIEW_TOOLBAR "View Toolbar"
#define ID_VIEW_BOOKMARKSBAR "View BookmarksBar"
#define ID_VIEW_FULLSCREEN "View Fullscreen"

static BonoboUIVerb galeon_verbs [] = {
        BONOBO_UI_VERB ("EditFind", (BonoboUIVerbFn)window_cmd_edit_find),
	BONOBO_UI_VERB ("FilePrint", (BonoboUIVerbFn)window_cmd_file_print),
	BONOBO_UI_VERB ("GoStop", (BonoboUIVerbFn)window_cmd_go_stop),
	BONOBO_UI_VERB ("GoReload", (BonoboUIVerbFn)window_cmd_go_reload),
	BONOBO_UI_VERB ("GoBack", (BonoboUIVerbFn)window_cmd_go_back),
	BONOBO_UI_VERB ("GoForward", (BonoboUIVerbFn)window_cmd_go_forward),
	BONOBO_UI_VERB ("GoGo", (BonoboUIVerbFn)window_cmd_go_go),
	BONOBO_UI_VERB ("GoUp", (BonoboUIVerbFn)window_cmd_go_up),
	BONOBO_UI_VERB ("GoHome", (BonoboUIVerbFn)window_cmd_go_home),
	BONOBO_UI_VERB ("GoMyportal", (BonoboUIVerbFn)window_cmd_go_myportal),
	BONOBO_UI_VERB ("GoLocation", (BonoboUIVerbFn)window_cmd_go_location),
	BONOBO_UI_VERB ("FileNewWindow", (BonoboUIVerbFn)window_cmd_new_window),
	BONOBO_UI_VERB ("FileNewTab", (BonoboUIVerbFn)window_cmd_new_tab),
	BONOBO_UI_VERB ("FileOpen", (BonoboUIVerbFn)window_cmd_file_open),
	BONOBO_UI_VERB ("FileSaveAs", (BonoboUIVerbFn)window_cmd_file_save_as),
	BONOBO_UI_VERB ("FileCloseTab", (BonoboUIVerbFn)window_cmd_file_close_tab),
	BONOBO_UI_VERB ("FileCloseWindow", (BonoboUIVerbFn)window_cmd_file_close_window),
	BONOBO_UI_VERB ("FileExit", (BonoboUIVerbFn)window_cmd_file_exit),
	BONOBO_UI_VERB ("FileSendTo", (BonoboUIVerbFn)window_cmd_file_send_to),
	BONOBO_UI_VERB ("EditCut", (BonoboUIVerbFn)window_cmd_edit_cut),
	BONOBO_UI_VERB ("EditCopy", (BonoboUIVerbFn)window_cmd_edit_copy),
	BONOBO_UI_VERB ("EditPaste", (BonoboUIVerbFn)window_cmd_edit_paste),
	BONOBO_UI_VERB ("EditSelectAll", (BonoboUIVerbFn)window_cmd_edit_select_all),
	BONOBO_UI_VERB ("EditPrefs", (BonoboUIVerbFn)window_cmd_edit_prefs),
	BONOBO_UI_VERB ("SettingsToolbarEditor", (BonoboUIVerbFn)window_cmd_settings_toolbar_editor),
	BONOBO_UI_VERB ("Zoom In", (BonoboUIVerbFn)window_cmd_view_zoom_in),
	BONOBO_UI_VERB ("EditFindNext", (BonoboUIVerbFn)window_cmd_edit_find_next),
	BONOBO_UI_VERB ("EditFindPrev", (BonoboUIVerbFn)window_cmd_edit_find_prev),
	BONOBO_UI_VERB ("Zoom Out", (BonoboUIVerbFn)window_cmd_view_zoom_out),
	BONOBO_UI_VERB ("Zoom Normal", (BonoboUIVerbFn)window_cmd_view_zoom_normal),
	BONOBO_UI_VERB ("ViewPageSource", (BonoboUIVerbFn)window_cmd_view_page_source),
	BONOBO_UI_VERB ("BookmarksAddDefault", (BonoboUIVerbFn)window_cmd_bookmarks_add_default),
	BONOBO_UI_VERB ("BookmarksEdit", (BonoboUIVerbFn)window_cmd_bookmarks_edit),
	BONOBO_UI_VERB ("ToolsHistory", (BonoboUIVerbFn)window_cmd_tools_history),
	BONOBO_UI_VERB ("ToolsPDM", (BonoboUIVerbFn)window_cmd_tools_pdm),
	BONOBO_UI_VERB ("TabsNext", (BonoboUIVerbFn)window_cmd_tabs_next),
	BONOBO_UI_VERB ("TabsPrevious", (BonoboUIVerbFn)window_cmd_tabs_previous),
	BONOBO_UI_VERB ("TabsMoveLeft", (BonoboUIVerbFn)window_cmd_tabs_move_left),
	BONOBO_UI_VERB ("TabsMoveRight", (BonoboUIVerbFn)window_cmd_tabs_move_right),
	BONOBO_UI_VERB ("HelpContents", (BonoboUIVerbFn)window_cmd_help_manual),
	BONOBO_UI_VERB ("About", (BonoboUIVerbFn)window_cmd_help_about),

        BONOBO_UI_VERB_END
};

static BonoboUIVerb galeon_popup_verbs [] = {
        BONOBO_UI_VERB ("EPOpenInNewWindow", (BonoboUIVerbFn)popup_cmd_new_window),
	BONOBO_UI_VERB ("EPOpenInNewTab", (BonoboUIVerbFn)popup_cmd_new_tab),
	BONOBO_UI_VERB ("EPAddBookmark", (BonoboUIVerbFn)popup_cmd_add_bookmark),
	BONOBO_UI_VERB ("EPOpenImageInNewWindow", (BonoboUIVerbFn)popup_cmd_image_in_new_window),
	BONOBO_UI_VERB ("EPOpenImageInNewTab", (BonoboUIVerbFn)popup_cmd_image_in_new_tab),
	BONOBO_UI_VERB ("DPOpenFrameInNewWindow", (BonoboUIVerbFn)popup_cmd_frame_in_new_window),
	BONOBO_UI_VERB ("DPOpenFrameInNewTab", (BonoboUIVerbFn)popup_cmd_frame_in_new_tab),
	BONOBO_UI_VERB ("DPAddFrameBookmark", (BonoboUIVerbFn)popup_cmd_add_frame_bookmark),
	BONOBO_UI_VERB ("DPViewSource", (BonoboUIVerbFn)popup_cmd_view_source),
	
        BONOBO_UI_VERB_END
};

struct GaleonWindowPrivate
{
	PPViewToolbar *ppview_toolbar;
	Toolbar *toolbar;
	Statusbar *statusbar;
	GbBonoboUIMenu *bookmarks_menu;
	GbBonoboUIToolbar *bookmarks_toolbar;
	GtkNotebook *notebook;
	GtkWidget *hpaned;
	GaleonTab *active_tab;
	GtkWidget *sidebar;
	GaleonEmbedPopupBW *embed_popup;
	GaleonDialog *find_dialog;
	GaleonDialog *history_dialog;
	GaleonDialog *history_sidebar;
	GbEditorDockable *bookmarks_sidebar;
	EmbedChromeMask chrome_mask;
	gboolean ignore_layout_toggles;
	gboolean has_default_size;
	gboolean closing;
	GaleonWindowRecentHistory *recent_history;
	GaleonWindowRecentHistoryMenu *recent_history_menu;
};

static void
galeon_window_class_init (GaleonWindowClass *klass);
static void
galeon_window_gb_location_source_init (GbLocationSourceIface *iface);
static void
galeon_window_init (GaleonWindow *gs);
static void
galeon_window_finalize (GObject *object);
static void
galeon_window_show (GtkWidget *widget);
static void
galeon_window_notebook_switch_page_cb (GtkNotebook *notebook,
				       GtkNotebookPage *page,
				       guint page_num,
				       GaleonWindow *window);
static void
galeon_window_bookmark_activated_cb (GObject *sender,
				     GbBookmark *b, const gchar *url, 
				     GbBookmarkActivatedFlags f, GaleonWindow *w);
void /* yes, this is not static */
galeon_window_bookmark_activate (GaleonWindow *w, GbBookmark *b, const gchar *url, 
				 GbBookmarkActivatedFlags f);

static char *
galeon_window_gb_location_source_get_location (GbLocationSource *src);
static char *
galeon_window_gb_location_source_get_title (GbLocationSource *src);

static void
galeon_window_recent_history_menu_activated_cb (GaleonWindowRecentHistoryMenu *wrhm, 
						const char *url, const char *title, 
						GaleonWindow *w);

static GObjectClass *parent_class = NULL;

GType 
galeon_window_get_type (void)
{
        static GType galeon_window_type = 0;

        if (galeon_window_type == 0)
        {
                static const GTypeInfo our_info =
			{
				sizeof (GaleonWindowClass),
				NULL, /* base_init */
				NULL, /* base_finalize */
				(GClassInitFunc) galeon_window_class_init,
				NULL,
				NULL, /* class_data */
				sizeof (GaleonWindow),
				0, /* n_preallocs */
				(GInstanceInitFunc) galeon_window_init
			};
		
		static const GInterfaceInfo gb_location_source_info =
			{
				(GInterfaceInitFunc) galeon_window_gb_location_source_init,
				NULL,
				NULL
			};

                galeon_window_type = g_type_register_static (BONOBO_TYPE_WINDOW,
							     "GaleonWindow",
							     &our_info, 0);
		
		g_type_add_interface_static (galeon_window_type,
					     GB_TYPE_LOCATION_SOURCE,
					     &gb_location_source_info);

        }

        return galeon_window_type;
}

static void
galeon_window_class_init (GaleonWindowClass *klass)
{
        GObjectClass *object_class = G_OBJECT_CLASS (klass);
	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
	
        parent_class = g_type_class_peek_parent (klass);

        object_class->finalize = galeon_window_finalize;
	
	widget_class->show = galeon_window_show;
}

static void
galeon_window_gb_location_source_init (GbLocationSourceIface *iface)
{
	iface->get_location = galeon_window_gb_location_source_get_location;
	iface->get_title = galeon_window_gb_location_source_get_title;
}

static
gboolean 
galeon_window_key_press_event_cb (GtkWidget *widget,
				  GdkEventKey *event,
				  GaleonWindow *window)
{
        int page;

	if ((event->state & GDK_Shift_L) || (event->state & GDK_Shift_R))
                return FALSE;

        if ((event->state & GDK_Alt_L) || (event->state & GDK_Alt_R))
        {
                page = event->keyval - GDK_0 -1;

                if (page == -1) page = 9;

                if (page>=-1 && page<=9)
                {
                        gtk_notebook_set_current_page 
				(GTK_NOTEBOOK (window->priv->notebook),
                                 page == -1 ? -1 : page);
                        return TRUE;
                }
        }
	
        return FALSE;
}

static void
galeon_window_selection_received_cb (GtkWidget *widget,
				     GtkSelectionData *selection_data,
				     guint time, GaleonWindow *window)
{
	GaleonTab *tab;

	if (selection_data->length <= 0 || selection_data->data == NULL)
		return;

	tab = galeon_window_get_active_tab (window);
	galeon_shell_new_tab (galeon_shell, window, tab,
			      selection_data->data, 0);
}
	
static void
save_window_state (GtkWidget *win)
{
	GaleonWindow *window = GALEON_WINDOW (win);
	
	if (!(window->priv->chrome_mask & EMBED_CHROME_OPENASPOPUP) &&
	    !(window->priv->chrome_mask & EMBED_CHROME_OPENASFULLSCREEN))
	{
		gul_state_save_window (win, "main_window");
	}
}

static gboolean
galeon_window_configure_event_cb (GtkWidget *widget,
				  GdkEventConfigure *event,
				  gpointer data)
{	
	save_window_state (widget);
	return FALSE;
}

static gboolean
galeon_window_state_event_cb (GtkWidget *widget,
			      GdkEventWindowState *event,
			      gpointer data)
{
	save_window_state (widget);
	return FALSE;
}

static void
setup_bonobo_window (GaleonWindow *window,
		     BonoboUIComponent **ui_component)
{
	BonoboWindow *win = BONOBO_WINDOW(window);
	BonoboUIContainer *container;
	Bonobo_UIContainer corba_container;

	container = bonobo_window_get_ui_container (win);
        
        bonobo_ui_engine_config_set_path (bonobo_window_get_ui_engine (win),
                                          "/apps/galeon/UIConfig/kvps");

        corba_container = BONOBO_OBJREF (container);

	*ui_component = bonobo_ui_component_new_default ();
	
	bonobo_ui_component_set_container (*ui_component, 
					   corba_container, 
					   NULL);

	bonobo_ui_util_set_ui (*ui_component,
			       DATADIR,
#ifdef ENABLE_SIMPLE_MENUBAR
			       "galeon-simple-ui.xml",
#else
			       "galeon-ui.xml",
#endif
			       "galeon", NULL);

	/* Key handling */
	g_signal_connect(G_OBJECT(win), 
			 "key-press-event",
                         G_CALLBACK(galeon_window_key_press_event_cb),
                         window);

	g_signal_connect (G_OBJECT(win), "configure_event",
			  G_CALLBACK (galeon_window_configure_event_cb), NULL);
	g_signal_connect (G_OBJECT(win), "window_state_event",
			  G_CALLBACK (galeon_window_state_event_cb), NULL);
	
	g_signal_connect (G_OBJECT(win),
			  "selection-received",
			  G_CALLBACK (galeon_window_selection_received_cb), 
			  window);
}

static GaleonEmbedPopupBW *
setup_popup_factory (GaleonWindow *window, 
		     BonoboUIComponent *ui_component)
{
	GaleonEmbedPopupBW *popup;
	
	popup = galeon_window_get_popup_factory (window);
	g_object_set_data (G_OBJECT(popup), "GaleonWindow", window);
	bonobo_ui_component_add_verb_list_with_data (ui_component, 
                                		     galeon_popup_verbs, 
						     popup);

	return popup;
}

static GtkNotebook *
setup_notebook (GaleonWindow *window)
{
	GtkNotebook *notebook;
	
	notebook = GTK_NOTEBOOK (gul_notebook_new ());
	gtk_notebook_set_scrollable (notebook, TRUE);
	gtk_notebook_set_show_border (notebook, FALSE);
	gtk_notebook_set_show_tabs (notebook, FALSE);

	g_signal_connect_after (GTK_OBJECT (notebook), "switch_page",
				GTK_SIGNAL_FUNC (
				galeon_window_notebook_switch_page_cb),
				window);
	gtk_widget_show (GTK_WIDGET (notebook));

	return notebook;
}

static GbBonoboUIMenu *
setup_bookmarks_menu (GaleonWindow *window, GbBookmarkSet *bs)
{
	GbBonoboUIMenu *menu;
	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT (window->ui_component);
	
	menu = gb_bonobo_ui_menu_new (bs->root, ui_component, 
				      "/menu/Bookmarks/BookmarksPlaceholder");
	gb_bonobo_ui_menu_set_location_source (menu, GB_LOCATION_SOURCE (window));

	g_signal_connect (menu, "bookmark-activated", 
			  G_CALLBACK (galeon_window_bookmark_activated_cb), window);

	return menu;
}

static GbBonoboUIToolbar *
setup_bookmarks_toolbar (GaleonWindow *window, GbBookmarkSet *bs)
{
	GbBonoboUIToolbar *toolbar;
	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT (window->ui_component);
	
	toolbar = gb_bonobo_ui_toolbar_new (bs, ui_component);
	gb_bonobo_ui_toolbar_set_location_source (toolbar, GB_LOCATION_SOURCE (window));
	
	g_signal_connect (toolbar, "bookmark-activated", 
			  G_CALLBACK (galeon_window_bookmark_activated_cb), window);

	return toolbar;
}

static void
view_toolbar_changed_cb (BonoboUIComponent *component,
                         const char *path,
                         Bonobo_UIComponent_EventType type,
                         const char *state,
                         GaleonWindow *window)
{
	EmbedChromeMask mask;

	if (window->priv->ignore_layout_toggles) return;
	
        mask = galeon_window_get_chrome (window);
        mask ^= EMBED_CHROME_TOOLBARON;
        galeon_window_set_chrome (window, mask);
}

static void
view_sidebar_changed_cb (BonoboUIComponent *component,
                         const char *path,
                         Bonobo_UIComponent_EventType type,
                         const char *state,
                         GaleonWindow *window)
{
	EmbedChromeMask mask;

	if (window->priv->ignore_layout_toggles) return;
	
        mask = galeon_window_get_chrome (window);
        mask ^= EMBED_CHROME_SIDEBARON;
        galeon_window_set_chrome (window, mask);
}

static void
view_statusbar_changed_cb (BonoboUIComponent *component,
                           const char *path,
                           Bonobo_UIComponent_EventType type,
                           const char *state,
                           GaleonWindow *window)
{
	EmbedChromeMask mask;

	if (window->priv->ignore_layout_toggles) return;
	
        mask = galeon_window_get_chrome (window);
        mask ^= EMBED_CHROME_STATUSBARON;
        galeon_window_set_chrome (window, mask);
}

static void
view_fullscreen_changed_cb (BonoboUIComponent *component,
                            const char *path,
                            Bonobo_UIComponent_EventType type,
                            const char *state,
                            GaleonWindow *window)
{	
	EmbedChromeMask mask;
	
	if (window->priv->ignore_layout_toggles) return;
	
	mask = galeon_window_get_chrome (window);
	mask ^= EMBED_CHROME_OPENASFULLSCREEN;
	mask |= EMBED_CHROME_DEFAULT;
	galeon_window_set_chrome (window, mask);
}

static void
update_layout_toggles (GaleonWindow *window)
{
	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT (window->ui_component);
	EmbedChromeMask mask = window->priv->chrome_mask;
	
	window->priv->ignore_layout_toggles = TRUE;

	gul_bonobo_set_toggle_state (ui_component, VIEW_SIDEBAR_PATH, 
				     mask & EMBED_CHROME_SIDEBARON);
	gul_bonobo_set_toggle_state (ui_component, VIEW_TOOLBAR_PATH, 
				     mask & EMBED_CHROME_TOOLBARON);
	gul_bonobo_set_toggle_state (ui_component, VIEW_STATUSBAR_PATH, 
				     mask & EMBED_CHROME_STATUSBARON);
	gul_bonobo_set_toggle_state (ui_component, VIEW_FULLSCREEN_PATH, 
				     mask & EMBED_CHROME_OPENASFULLSCREEN);

	window->priv->ignore_layout_toggles = FALSE;
}

static void
setup_layout_menus (GaleonWindow *window)
{
	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT (window->ui_component);
	
	bonobo_ui_component_add_listener (ui_component, ID_VIEW_SIDEBAR, 
					  (BonoboUIListenerFn)view_sidebar_changed_cb, 
					  window);
	bonobo_ui_component_add_listener (ui_component, ID_VIEW_TOOLBAR, 
					  (BonoboUIListenerFn)view_toolbar_changed_cb, 
					  window);
	bonobo_ui_component_add_listener (ui_component, ID_VIEW_STATUSBAR, 
					  (BonoboUIListenerFn)view_statusbar_changed_cb, 
					  window);
	bonobo_ui_component_add_listener (ui_component, ID_VIEW_FULLSCREEN, 
					  (BonoboUIListenerFn)view_fullscreen_changed_cb, 
					  window);
}

static void
sidebar_close_requested_cb (GaleonSidebar *sidebar, 
			    GaleonWindow *window)
{
	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT (window->ui_component);
	
	gul_bonobo_set_toggle_state (ui_component, VIEW_SIDEBAR_PATH, 
				     FALSE);
}

static void
sidebar_page_changed_cb (GaleonSidebar *sidebar, 
			 const char *page_id, 
			 GaleonWindow *window)
{
	GaleonEmbed *embed;

	embed = galeon_window_get_active_embed (window);

	if (strcmp (page_id, "history") == 0)
	{
		window->priv->history_sidebar = 
			history_dialog_new (embed, TRUE);
	
		galeon_sidebar_set_content 
			(sidebar, 
			 G_OBJECT(window->priv->history_sidebar));
	}
	else if (strcmp (page_id, "bookmarks") == 0)
	{
		GtkWidget *w;
		if (!window->priv->bookmarks_sidebar)
		{
			GbEditorDockable *e;
			GbBookmarkSet *set;
			
			set = galeon_shell_get_bookmark_set (galeon_shell);
			if (!set) return;
		
			e = gb_editor_dockable_new_for_set (set);
			
			gb_editor_set_location_source (GB_EDITOR (e), GB_LOCATION_SOURCE (window));

			g_signal_connect (e, "bookmark-activated", 
					  G_CALLBACK (galeon_window_bookmark_activated_cb), window);
			window->priv->bookmarks_sidebar = e;
		}
		w = gb_editor_get_main_widget (GB_EDITOR (window->priv->bookmarks_sidebar));
		gtk_widget_show (w);
		galeon_sidebar_set_content (sidebar, G_OBJECT (w));
	}
}

static GtkWidget *
setup_sidebar (GaleonWindow *window)
{
	GtkWidget *hpaned;
	GtkWidget *sidebar;

	hpaned = gtk_hpaned_new ();
	gtk_widget_show (hpaned);

	sidebar = galeon_sidebar_new ();

	g_signal_connect (G_OBJECT(sidebar),
			  "close-requested",
			  G_CALLBACK (sidebar_close_requested_cb),
			  window);
	
	galeon_sidebar_add_page (GALEON_SIDEBAR(sidebar), 
				 _("Bookmarks"), "bookmarks", FALSE);
	galeon_sidebar_add_page (GALEON_SIDEBAR(sidebar), 
				 _("History"), "history", FALSE);

	g_signal_connect (sidebar, "page_changed",
			  G_CALLBACK (sidebar_page_changed_cb),
			  window);

	gtk_paned_add1 (GTK_PANED(hpaned), 
			sidebar);
	gtk_paned_add2 (GTK_PANED(hpaned),
			GTK_WIDGET(window->priv->notebook));

	window->priv->sidebar = sidebar;
	
	return hpaned;
}

static void
galeon_window_init (GaleonWindow *window)
{
	BonoboUIComponent *ui_component;
	GbBookmarkSet *bs;
	Session *session;

	session = galeon_shell_get_session (galeon_shell);

        window->priv = g_new0 (GaleonWindowPrivate, 1);
	window->priv->embed_popup = NULL;
	window->priv->active_tab = NULL;
	window->priv->chrome_mask = 0;
	window->priv->ignore_layout_toggles = FALSE;
	window->priv->closing = FALSE;
	window->priv->has_default_size = FALSE;
	window->priv->recent_history = galeon_window_recent_history_new ();
	window->priv->recent_history_menu = galeon_window_recent_history_menu_new ();

	galeon_window_recent_history_menu_set_history (window->priv->recent_history_menu,
						       window->priv->recent_history);
	g_signal_connect (window->priv->recent_history_menu, "activated",
			  G_CALLBACK (galeon_window_recent_history_menu_activated_cb), window);

	/* Setup the window and connect verbs */
	setup_bonobo_window (window, &ui_component);
	window->ui_component = G_OBJECT (ui_component);
	bonobo_ui_component_add_verb_list_with_data (ui_component, 
                                		     galeon_verbs, 
						     window);
	setup_layout_menus (window);
	
	/* Setup the embed popups factory */
	window->priv->embed_popup = setup_popup_factory (window, 
							 ui_component);

	bonobo_ui_component_freeze (ui_component, NULL);
	
	/* Setup menubar */
	galeon_embed_utils_build_charsets_submenu (ui_component,
                                                   CHARSET_MENU_PATH,
                                                   (BonoboUIVerbFn)window_cmd_set_charset,
                                                   window);
	galeon_window_recent_history_menu_set_path (window->priv->recent_history_menu, 
						    ui_component, GO_RECENT_HISTORY_PATH);

	/* Setup toolbar and statusbar */
	window->priv->toolbar = toolbar_new (window);
	window->priv->statusbar = statusbar_new (window);
	window->priv->ppview_toolbar = ppview_toolbar_new (window);

	/* Setup bookmarks ui elements */
	bs = galeon_shell_get_bookmark_set (galeon_shell);
	window->priv->bookmarks_menu = setup_bookmarks_menu (window, bs); 
	window->priv->bookmarks_toolbar = setup_bookmarks_toolbar (window, bs); 	
	
	/* Setup window contents */
	window->priv->notebook = setup_notebook (window);
	window->priv->hpaned = setup_sidebar (window);
	bonobo_window_set_contents (BONOBO_WINDOW (window), 
				    GTK_WIDGET (window->priv->hpaned));
	
	bonobo_ui_component_thaw (ui_component, NULL);
	
	g_object_ref (galeon_shell);

	/*Once window is fully created, add it to the session list*/
	session_add_window (session, window);
}

static void  
save_window_chrome (GaleonWindow *window)
{
	EmbedChromeMask flags = window->priv->chrome_mask;
	
	if (flags & EMBED_CHROME_OPENASPOPUP)
	{
	}
	else if (flags & EMBED_CHROME_PPVIEWTOOLBARON)
	{
	}
	else if (flags & EMBED_CHROME_OPENASFULLSCREEN)
	{
		eel_gconf_set_boolean (CONF_WINDOWS_FS_SHOW_SIDEBAR,
				       flags & EMBED_CHROME_SIDEBARON);
		eel_gconf_set_boolean (CONF_WINDOWS_FS_SHOW_TOOLBARS,
				       flags & EMBED_CHROME_TOOLBARON);
		eel_gconf_set_boolean (CONF_WINDOWS_FS_SHOW_STATUSBAR,
				       flags & EMBED_CHROME_STATUSBARON);
	}
	else
	{
		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_SIDEBAR,
				       flags & EMBED_CHROME_SIDEBARON);
		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_TOOLBARS,
				       flags & EMBED_CHROME_TOOLBARON);
		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_STATUSBAR,
				       flags & EMBED_CHROME_STATUSBARON);		
	}
}

static void
remove_from_session (GaleonWindow *window)
{
	Session *session;

	session = galeon_shell_get_session (galeon_shell);
	g_return_if_fail (session != NULL);
	
	session_remove_window (session, window);
}

static void
galeon_window_finalize (GObject *object)
{
        GaleonWindow *window;

        g_return_if_fail (IS_GALEON_WINDOW (object));

	window = GALEON_WINDOW (object);

        g_return_if_fail (window->priv != NULL);

	remove_from_session (window);

	if (window->priv->embed_popup)
	{
		g_object_unref (G_OBJECT (window->priv->embed_popup));
	}

	g_object_unref (G_OBJECT (window->priv->bookmarks_menu));
	g_object_unref (G_OBJECT (window->priv->bookmarks_toolbar));
	g_object_unref (G_OBJECT (window->priv->toolbar));
	g_object_unref (G_OBJECT (window->priv->statusbar));
	
	if (window->priv->find_dialog)
	{
		g_object_unref (G_OBJECT (window->priv->find_dialog));
	}

	if (window->priv->history_dialog)
	{
		g_object_remove_weak_pointer 
                        (G_OBJECT(window->priv->history_dialog),
                         (gpointer *)&window->priv->history_dialog);
	}

	if (window->priv->bookmarks_sidebar)
	{
		g_object_unref (window->priv->bookmarks_sidebar);
	}
	
	g_signal_handlers_disconnect_matched (window->priv->recent_history_menu, 
					      G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, window);

	g_object_unref (window->priv->recent_history);
	g_object_unref (window->priv->recent_history_menu);

        g_free (window->priv);

        G_OBJECT_CLASS (parent_class)->finalize (object);

#ifdef DEBUG_MARCO
	g_print ("Galeon Window finalized %p\n", window);
#endif
	
	g_object_unref (galeon_shell);
}

GaleonWindow *
galeon_window_new (void)
{
	return GALEON_WINDOW (g_object_new (GALEON_WINDOW_TYPE, NULL));
}

static void
dock_item_set_visibility (GaleonWindow *window,
			  const char *path,
			  gboolean visibility)
{
	gul_bonobo_set_hidden (BONOBO_UI_COMPONENT(window->ui_component),
                               path,
                               !visibility);
}

EmbedChromeMask  
galeon_window_get_chrome (GaleonWindow *window)
{
	return window->priv->chrome_mask;
}

static void
wmspec_change_state (gboolean   add,
                     GdkWindow *window,
                     GdkAtom    state1,
                     GdkAtom    state2)
{
	XEvent xev;

	#define _NET_WM_STATE_REMOVE        0    /* remove/unset property */
	#define _NET_WM_STATE_ADD           1    /* add/set property */
	#define _NET_WM_STATE_TOGGLE        2    /* toggle property  */  
  
	xev.xclient.type = ClientMessage;
  	xev.xclient.serial = 0;
  	xev.xclient.send_event = True;
  	xev.xclient.display = gdk_display;
  	xev.xclient.window = GDK_WINDOW_XID (window);
  	xev.xclient.message_type = gdk_x11_get_xatom_by_name ("_NET_WM_STATE");
  	xev.xclient.format = 32;
  	xev.xclient.data.l[0] = add ? _NET_WM_STATE_ADD : _NET_WM_STATE_REMOVE;
  	xev.xclient.data.l[1] = gdk_x11_atom_to_xatom (state1);
  	xev.xclient.data.l[2] = gdk_x11_atom_to_xatom (state2);
  
  	XSendEvent (gdk_display, GDK_WINDOW_XID (gdk_get_default_root_window ()),
                    False,
                    SubstructureRedirectMask | SubstructureNotifyMask,
                    &xev);
}

static void
window_set_fullscreen_mode (GaleonWindow *window, gboolean active)
{
        GdkWindow *gdk_window;

        gdk_window = GTK_WIDGET(window)->window;

        if (gdk_net_wm_supports (gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN",
                                                  FALSE)))
        {
                wmspec_change_state (active,
                                     gdk_window,
                                     gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN",
                                                      FALSE),
                                     GDK_NONE);
        }
        else
        {
                g_warning ("NET_WM_STATE_FULLSCREEN not supported");
	}
}

static void
translate_default_chrome (EmbedChromeMask *chrome_mask)
{	
	/* keep only not layout flags */
	*chrome_mask &= (EMBED_CHROME_WINDOWRAISED |
        		 EMBED_CHROME_WINDOWLOWERED |
        		 EMBED_CHROME_CENTERSCREEN |
        		 EMBED_CHROME_OPENASDIALOG |
        		 EMBED_CHROME_OPENASCHROME |
        		 EMBED_CHROME_OPENASPOPUP |
			 EMBED_CHROME_OPENASFULLSCREEN);

	/* Load defaults */
	if (*chrome_mask & EMBED_CHROME_OPENASFULLSCREEN)
	{
		if (eel_gconf_get_boolean (CONF_WINDOWS_FS_SHOW_SIDEBAR))
		{
			*chrome_mask |= EMBED_CHROME_SIDEBARON;
		}
		if (eel_gconf_get_boolean (CONF_WINDOWS_FS_SHOW_STATUSBAR))
		{
			*chrome_mask |= EMBED_CHROME_STATUSBARON;
		}
		if (eel_gconf_get_boolean (CONF_WINDOWS_FS_SHOW_TOOLBARS))
		{
			*chrome_mask |= EMBED_CHROME_TOOLBARON;
		}
		if (eel_gconf_get_boolean (CONF_WINDOWS_FS_SHOW_BOOKMARKS))
		{
			*chrome_mask |= EMBED_CHROME_PERSONALTOOLBARON;
		}
	}
	else
	{
		if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_SIDEBAR))
		{
			*chrome_mask |= EMBED_CHROME_SIDEBARON;
		}
		if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_STATUSBAR))
		{
			*chrome_mask |= EMBED_CHROME_STATUSBARON;
		}
		if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_TOOLBARS))
		{
			*chrome_mask |= EMBED_CHROME_TOOLBARON;
		}
		
		*chrome_mask |= EMBED_CHROME_PERSONALTOOLBARON;
		*chrome_mask |= EMBED_CHROME_MENUBARON;
	}
}

void  
galeon_window_set_chrome (GaleonWindow *window, 
			  EmbedChromeMask flags)
{
	gboolean toolbar, ppvtoolbar, statusbar;
	
	if (flags & EMBED_CHROME_DEFAULT)
	{
		translate_default_chrome (&flags);
	}
		
	dock_item_set_visibility (window, "/menu", 
				  flags & EMBED_CHROME_MENUBARON);	

	if ((flags & EMBED_CHROME_SIDEBARON) != FALSE)
	{
		gtk_widget_show (window->priv->sidebar);
	}
	else
	{
		gtk_widget_hide (window->priv->sidebar);
	}
	
	toolbar = (flags & EMBED_CHROME_TOOLBARON) != FALSE;
	toolbar_set_visibility (window->priv->toolbar,
				toolbar);

	statusbar = (flags & EMBED_CHROME_STATUSBARON) != FALSE;
	statusbar_set_visibility (window->priv->statusbar,
				  statusbar);

	ppvtoolbar = (flags & EMBED_CHROME_PPVIEWTOOLBARON) != FALSE;
	ppview_toolbar_set_old_chrome (window->priv->ppview_toolbar,
				       window->priv->chrome_mask);
	ppview_toolbar_set_visibility (window->priv->ppview_toolbar, 
				       ppvtoolbar);
	
	/* set fullscreen only when it's really changed */
	if ((window->priv->chrome_mask & EMBED_CHROME_OPENASFULLSCREEN) != 
	    (flags & EMBED_CHROME_OPENASFULLSCREEN))
	{
		save_window_chrome (window);
		window_set_fullscreen_mode (window,
					    flags & EMBED_CHROME_OPENASFULLSCREEN);
	}

	gb_bonobo_ui_toolbar_set_visible (window->priv->bookmarks_toolbar,
					  flags & EMBED_CHROME_PERSONALTOOLBARON);
	
	window->priv->chrome_mask = flags;

	update_layout_toggles (window);

	save_window_chrome (window);
}

GtkWidget *
galeon_window_get_notebook (GaleonWindow *window)
{
	return GTK_WIDGET (window->priv->notebook);
}

void 
galeon_window_add_tab (GaleonWindow *window, 
		       GaleonTab *tab,
		       gboolean jump_to)
{
	GtkWidget *widget;
	
	g_return_if_fail (IS_GALEON_WINDOW (window));
	g_return_if_fail (IS_GALEON_TAB (tab));

	galeon_tab_set_window (tab, window);
	
	widget = GTK_WIDGET(galeon_tab_get_embed (tab));

	gul_notebook_insert_page (GUL_NOTEBOOK (window->priv->notebook), 
				  widget,
				  -1,
				  jump_to);
}

void 
galeon_window_jump_to_tab (GaleonWindow *window, 
			   GaleonTab *tab)
{
	GtkWidget *widget;	
	int page;
	
	widget = GTK_WIDGET(galeon_tab_get_embed (tab));

	page = gtk_notebook_page_num 
		(window->priv->notebook, widget);
	gtk_notebook_set_current_page 
		(window->priv->notebook, page);
}

static GaleonTab *
real_get_active_tab (GaleonWindow *window, int page_num)
{
	GaleonTab *tab;	
	GtkWidget *embed_widget;

	if (page_num == -1)
	{
		page_num = gtk_notebook_get_current_page (window->priv->notebook);
	}

	if (page_num < 0) return NULL;
	
	embed_widget = gtk_notebook_get_nth_page 
		(window->priv->notebook, page_num);
		
	g_return_val_if_fail (GTK_IS_WIDGET (embed_widget), NULL);
	tab = g_object_get_data (G_OBJECT (embed_widget), "GaleonTab");
	g_return_val_if_fail (IS_GALEON_TAB (G_OBJECT (tab)), NULL);

	return tab;
}

void 
galeon_window_remove_tab (GaleonWindow *window, 
		          GaleonTab *tab)
{
	GtkWidget *embed;
	
	g_return_if_fail (IS_GALEON_WINDOW (window));
	g_return_if_fail (IS_GALEON_TAB (tab));

	window->priv->active_tab = NULL;
	
	embed  = GTK_WIDGET (galeon_tab_get_embed (tab));
	
	gul_notebook_remove_page (GUL_NOTEBOOK (window->priv->notebook), 
				  embed);
}

void 
galeon_window_move_tab (GaleonWindow *window, 
			GaleonTab *tab,
			GaleonTab *new_sibling)
{
	int move_to;
	GtkNotebook *nb = window->priv->notebook;
	GtkWidget *w;
	GtkWidget *embed;
	
	embed = GTK_WIDGET(galeon_tab_get_embed (tab)); 
	w = GTK_WIDGET (galeon_tab_get_embed (new_sibling));
	move_to = gtk_notebook_page_num (nb, w);

	if (embed && move_to != -1)
	{
		gtk_notebook_reorder_child (nb, embed, move_to);
	}
}

void 
galeon_window_load_url (GaleonWindow *window,
			const char *url)
{
	GaleonEmbed *embed;
	GbBookmarkSet *set;
	gchar *real_url;

        g_return_if_fail (IS_GALEON_WINDOW(window));
	embed = galeon_window_get_active_embed (window);
        g_return_if_fail (embed != NULL);
        g_return_if_fail (url != NULL);
	
	set = galeon_shell_get_bookmark_set (galeon_shell);
	if (set)
	{
		real_url = gb_bookmark_set_get_url_by_nick_and_args (set, url);
		if (!real_url)
		{
			/* no matching nick found */
			real_url = g_strdup (url);
		}
	}
	else
	{
		real_url = g_strdup (url);
	}

        galeon_embed_load_url (embed, real_url);
	g_free (real_url);
}

void galeon_window_activate_location (GaleonWindow *window)
{
	toolbar_activate_location (window->priv->toolbar);
}

void
galeon_window_show (GtkWidget *widget)
{
	GaleonWindow *window = GALEON_WINDOW(widget);
	GaleonTab *tab;
	int w = -1, h = -1;
	
	if (!window->priv->chrome_mask)
	{
		galeon_window_set_chrome (window, EMBED_CHROME_DEFAULT);
	}

	tab = galeon_window_get_active_tab (window);
	if (tab) galeon_tab_get_size (tab, &w, &h);
	
	if (!window->priv->has_default_size && w == -1 && h == -1) 
	{
		gul_state_load_window (GTK_WIDGET(window), 
				       "main_window", 
			               600, 500, TRUE);
		window->priv->has_default_size = TRUE;
	}
	
	GTK_WIDGET_CLASS (parent_class)->show (widget);
}

static void
update_status_message (GaleonWindow *window)
{
	const char *message;
	GaleonTab *tab;

	tab = galeon_window_get_active_tab (window);
	g_return_if_fail (tab != NULL);
	
	message = galeon_tab_get_status_message (tab);
	g_return_if_fail (message != NULL);
	
	statusbar_set_message (STATUSBAR(window->priv->statusbar),
			       message);
}

static void
update_progress (GaleonWindow *window)
{
	GaleonTab *tab;
	int load_percent;
	
	tab = galeon_window_get_active_tab (window);
	g_return_if_fail (tab != NULL);
	
	load_percent = galeon_tab_get_load_percent (tab);

	statusbar_set_progress (STATUSBAR(window->priv->statusbar),
			        load_percent);
}

static void
update_security (GaleonWindow *window)
{
	GaleonEmbed *embed;
	EmbedSecurityLevel level;
	char *description;
	char *state = NULL;
	gboolean secure;
	char *tooltip;

	embed = galeon_window_get_active_embed (window);
	g_return_if_fail (embed != NULL);

	level = STATE_IS_INSECURE;
	description = NULL;
	galeon_embed_get_security_level (embed, &level, &description);

	secure = FALSE;
	switch (level)
	{
	case STATE_IS_UNKNOWN:
		state = _("Unknown");
		break;
	case STATE_IS_INSECURE:
		state = _("Insecure");
		break;
	case STATE_IS_BROKEN:
		state = _("Broken");
		break;
	case STATE_IS_SECURE_MED:
		state = _("Medium");
		secure = TRUE;
		break;
	case STATE_IS_SECURE_LOW:
		state = _("Low");
		secure = TRUE;
		break;
	case STATE_IS_SECURE_HIGH:
		state = _("High");
		secure = TRUE;
		break;
	default:
		g_assert_not_reached ();
		break;
	}

	if (description != NULL)
	{
		tooltip = g_strdup_printf (_("Security level: %s\n%s"),
					   state, description);
		g_free (description);
	}
	else
	{
		tooltip = g_strdup_printf (_("Security level: %s"), state);

	}

	statusbar_set_security_state (STATUSBAR (window->priv->statusbar),
			              secure, tooltip);
	g_free (tooltip);
}

static void
update_nav_control (GaleonWindow *window)
{
	/* the zoom control is updated at the same time than the navigation
	   controls. This keeps it synched most of the time, but not always,
	   because we don't get a notification when zoom changes */

	gresult back, forward, up, stop;
	GaleonEmbed *embed;
	GaleonTab *tab;
	gint zoom;

	g_return_if_fail (window != NULL);
	
	tab = galeon_window_get_active_tab (window);
	g_return_if_fail (tab != NULL);
	
	embed = galeon_window_get_active_embed (window);
	g_return_if_fail (embed != NULL);
	
	back = galeon_embed_can_go_back (embed);
	forward = galeon_embed_can_go_forward (embed);
	up = galeon_embed_can_go_up (embed);
	stop = galeon_tab_get_load_status (tab) & TAB_LOAD_STARTED;
	galeon_embed_zoom_get (embed, &zoom);

	toolbar_button_set_sensitive (window->priv->toolbar, 
				      TOOLBAR_BACK_BUTTON,
				      back == G_OK ? TRUE : FALSE);
	toolbar_button_set_sensitive (window->priv->toolbar, 
				      TOOLBAR_FORWARD_BUTTON,
				      forward == G_OK ? TRUE : FALSE);
	toolbar_button_set_sensitive (window->priv->toolbar, 
				      TOOLBAR_UP_BUTTON,
				      up == G_OK ? TRUE : FALSE);
	toolbar_button_set_sensitive (window->priv->toolbar, 
				      TOOLBAR_STOP_BUTTON,
				      stop);
	toolbar_set_zoom (window->priv->toolbar, zoom);
	

	gul_bonobo_set_sensitive (BONOBO_UI_COMPONENT(window->ui_component),
			          GO_BACK_CMD_PATH, !back);
	gul_bonobo_set_sensitive (BONOBO_UI_COMPONENT(window->ui_component),
			          GO_FORWARD_CMD_PATH, !forward);
	gul_bonobo_set_sensitive (BONOBO_UI_COMPONENT(window->ui_component),
			          GO_UP_CMD_PATH, !up);
}

static void
update_title_control (GaleonWindow *window)
{
	GaleonTab *tab;
	const char *title;
	
	tab = galeon_window_get_active_tab (window);
	g_return_if_fail (tab != NULL);

	title = galeon_tab_get_title (tab);

	if (title)
	{
		gtk_window_set_title (GTK_WINDOW(window), 
				      title);
	}
}

static void
update_location_control (GaleonWindow *window)
{
	GaleonTab *tab;
	const char *location;
	
	tab = galeon_window_get_active_tab (window);
	g_return_if_fail (tab != NULL);

	location = galeon_tab_get_location (tab);
	
	if (!location) location = "";

	toolbar_set_location (window->priv->toolbar,
			      location);
}

static void
update_favicon_control (GaleonWindow *window)
{
	toolbar_update_favicon (window->priv->toolbar);
}

static void
update_find_control (GaleonWindow *window)
{
	gboolean can_go_next, can_go_prev;

	if (window->priv->find_dialog)
	{
		can_go_next = find_dialog_can_go_next 
			(FIND_DIALOG(window->priv->find_dialog));
		can_go_prev = find_dialog_can_go_prev 
			(FIND_DIALOG(window->priv->find_dialog));
		
		gul_bonobo_set_sensitive (BONOBO_UI_COMPONENT(window->ui_component),
					  EDIT_FIND_NEXT_CMD_PATH, can_go_next);
		gul_bonobo_set_sensitive (BONOBO_UI_COMPONENT(window->ui_component),
					  EDIT_FIND_PREV_CMD_PATH, can_go_prev);
	}
}

static void
update_window_visibility (GaleonWindow *window)
{
	GList *l;
	
	l = galeon_window_get_tabs (window);
	for (; l != NULL; l = l->next)
	{
		GaleonTab *tab = GALEON_TAB(l->data);
		g_return_if_fail (IS_GALEON_TAB(tab));
		
		if (galeon_tab_get_visibility (tab))
		{
			gtk_widget_show (GTK_WIDGET(window));
			return;
		}
	}
	g_list_free (l);

	if (GTK_WIDGET_VISIBLE (GTK_WIDGET (window)))
	{
		gtk_widget_hide (GTK_WIDGET (window));
	}
}

static void
update_spinner_control (GaleonWindow *window)
{
	GList *l;

	l = galeon_window_get_tabs (window);
	for (; l != NULL; l = l->next)
	{
		GaleonTab *tab = GALEON_TAB(l->data);
		g_return_if_fail (IS_GALEON_TAB(tab));
		
		if (galeon_tab_get_load_status (tab) & TAB_LOAD_STARTED)
		{
			toolbar_spinner_start (window->priv->toolbar);
			return;
		}
	}
	g_list_free (l);
	
	toolbar_spinner_stop (window->priv->toolbar);
}

void  
galeon_window_update_control (GaleonWindow *window,
			      ControlID control)
{
	g_return_if_fail (IS_GALEON_WINDOW (window));
	
	switch (control)
	{
	case StatusbarMessageControl:
		update_status_message (window);
		break;
	case StatusbarProgressControl:
		update_progress (window);
		break;
	case StatusbarSecurityControl:
		update_security (window);
		break;
	case FindControl:
		update_find_control (window);
		break;
	case ZoomControl:
		 /* the zoom control is updated at the same time than the navigation
		    controls. This keeps it synched most of the time, but not always,
		    because we don't get a notification when zoom changes */
	case NavControl:
		update_nav_control (window);
		break;
	case TitleControl:
		update_title_control (window);
		break;
	case WindowVisibilityControl:
		update_window_visibility (window);
		break;
	case SpinnerControl:
		update_spinner_control (window);
		break;
	case LocationControl:
		update_location_control (window);
		break;
	case FaviconControl:
		update_favicon_control (window);
		break;
	default:
		g_warning ("unknown control specified for updating");
		break;
	}
}

void
galeon_window_update_all_controls (GaleonWindow *window)
{
	g_return_if_fail (IS_GALEON_WINDOW (window));
	
	if (galeon_window_get_active_tab (window) != NULL)
	{
		update_nav_control (window);
		update_title_control (window);
		update_location_control (window);
		update_favicon_control (window);
		update_status_message (window);
		update_progress (window);
		update_find_control (window);
		update_spinner_control (window);
	}
}


GaleonSidebar *
galeon_window_get_sidebar (GaleonWindow *window)
{
	g_return_val_if_fail (IS_GALEON_WINDOW (window), NULL);

	if(!IS_GALEON_SIDEBAR(G_OBJECT(window->priv->sidebar)))
		setup_sidebar(window);

	return GALEON_SIDEBAR(window->priv->sidebar);
}	

GaleonTab *
galeon_window_get_active_tab (GaleonWindow *window)
{
	g_return_val_if_fail (IS_GALEON_WINDOW (window), NULL);

	return window->priv->active_tab;
}

GaleonEmbed *
galeon_window_get_active_embed (GaleonWindow *window)
{
	GaleonTab *tab;

	tab = galeon_window_get_active_tab (window);

	if (tab)
	{
		g_return_val_if_fail (IS_GALEON_WINDOW (G_OBJECT (window)),
				      NULL);
		return galeon_tab_get_embed (tab);
	}
	else return NULL;
}

GaleonEmbedPopupBW *
galeon_window_get_popup_factory (GaleonWindow *window)
{
	if (!window->priv->embed_popup)
	{
		window->priv->embed_popup = galeon_embed_popup_bw_new 
			(BONOBO_WINDOW(window));
		galeon_embed_popup_connect_verbs
			(GALEON_EMBED_POPUP (window->priv->embed_popup),
			 BONOBO_UI_COMPONENT (window->ui_component));
	}
	
	return window->priv->embed_popup;
}

GList *
galeon_window_get_tabs (GaleonWindow *window)
{
	GList *tabs = NULL;
	GtkWidget *w;
	int i = 0;
	
	while ((w = gtk_notebook_get_nth_page (window->priv->notebook, i)) != NULL)
	{
		GaleonTab *tab;
	
		tab = g_object_get_data (G_OBJECT (w), "GaleonTab");
		g_return_val_if_fail (IS_GALEON_TAB (G_OBJECT (tab)), NULL);
		
		tabs = g_list_append (tabs, tab);
		i++;
	}

	return tabs;
}

static void
save_old_embed_status (GaleonTab *tab, GaleonWindow *window)
{
	/* save old tab location status */
	galeon_tab_set_location (tab, 
				 toolbar_get_location (window->priv->toolbar));
}

static void
update_embed_dialogs (GaleonWindow *window,
		      GaleonTab *tab)
{
	GaleonEmbed *embed;
	GaleonDialog *find_dialog = window->priv->find_dialog;
	GaleonDialog *history_dialog = window->priv->history_dialog;
	GaleonDialog *history_sidebar = window->priv->history_sidebar;

	embed = galeon_tab_get_embed (tab);

	if (find_dialog)
	{
		galeon_embed_dialog_set_embed 
			(GALEON_EMBED_DIALOG(find_dialog),
			 embed);
	}

	if (history_dialog)
	{
		galeon_embed_dialog_set_embed 
			(GALEON_EMBED_DIALOG(history_dialog),
			 embed);
	}

	if (history_sidebar)
	{
		galeon_embed_dialog_set_embed 
			(GALEON_EMBED_DIALOG(history_sidebar),
			 embed);
	}
}

static void
galeon_window_notebook_switch_page_cb (GtkNotebook *notebook,
				       GtkNotebookPage *page,
				       guint page_num,
				       GaleonWindow *window)
{
	GaleonTab *tab, *old_tab;

	if (window->priv->closing) return;
	
	g_return_if_fail (IS_GALEON_WINDOW (window));

	/* get the new tab */
	tab = real_get_active_tab (window, page_num);
	
	/* update old tab */
	old_tab = window->priv->active_tab;
	if (old_tab && tab != old_tab)
	{
		g_return_if_fail (IS_GALEON_TAB (G_OBJECT (old_tab)));
		galeon_tab_set_is_active (old_tab, FALSE);
		save_old_embed_status (old_tab, window);
	}

	/* update new tab */
	window->priv->active_tab = tab;
	galeon_tab_set_is_active (tab, TRUE);

	update_embed_dialogs (window, tab);
	
	/* update window controls */
	galeon_window_update_all_controls (window);
}

/**
 * Handle the activation of a bookmark. This function is not static because it's 
 * used in galeon-window-bookmark-activated-proxy.c (and only there: it's not public)
 */
void
galeon_window_bookmark_activate (GaleonWindow *w, GbBookmark *b, const gchar *url, 
				 GbBookmarkActivatedFlags f)
{
	GaleonTab *tab;
	tab = galeon_window_get_active_tab (w);

	if (GB_IS_SITE (b))
	{
		if (f == GB_BAF_DEFAULT)
		{
			galeon_window_load_url (w, url);
		}
		else if (f == GB_BAF_NEW_WINDOW)
		{
			galeon_shell_new_tab (galeon_shell, NULL, tab, url, 
					      GALEON_NEW_TAB_IN_NEW_WINDOW);
		} 
		else if (f == GB_BAF_NEW_TAB)
		{
			galeon_shell_new_tab (galeon_shell, w, tab, url, 
					      GALEON_NEW_TAB_IN_EXISTING_WINDOW);
		}
		else if (f == GB_BAF_NEW_TAB_OR_WINDOW)
		{
			galeon_shell_new_tab (galeon_shell, w, tab, url, 0);
		}
	}
	else if (GB_IS_FOLDER (b))
	{
		if (f == GB_BAF_NEW_TAB || f == GB_BAF_DEFAULT)
		{
			GbBookmark *bi;
			for (bi = GB_FOLDER (b)->child; bi; bi = bi->next)
			{
				if (GB_IS_SITE (bi))
				{
					tab = galeon_shell_new_tab (galeon_shell, w, tab,
								    GB_SITE (bi)->url, 
								    GALEON_NEW_TAB_IN_EXISTING_WINDOW);
				}
			}
		}
		else if (f == GB_BAF_NEW_WINDOW)
		{
			GbBookmark *bi;
			for (bi = GB_FOLDER (b)->child; bi; bi = bi->next)
			{
				if (GB_IS_SITE (bi))
				{
					galeon_shell_new_tab (galeon_shell, NULL, tab, GB_SITE (bi)->url, 
							      GALEON_NEW_TAB_IN_NEW_WINDOW);
				}
			}
		}
		else if (f == GB_BAF_NEW_TAB_OR_WINDOW)
		{
			GbBookmark *bi;
			for (bi = GB_FOLDER (b)->child; bi; bi = bi->next)
			{
				if (GB_IS_SITE (bi))
				{
					galeon_shell_new_tab (galeon_shell, w, tab, GB_SITE (bi)->url, 
							      0);
				}
			}
		}
	}
	else
	{
		g_warning ("don't know what to do...");
	}
}

static void
galeon_window_bookmark_activated_cb (GObject *sender,
				     GbBookmark *b, const gchar *url, 
				     GbBookmarkActivatedFlags f, GaleonWindow *w)
{
	galeon_window_bookmark_activate (w, b, url, f);
}

static void
find_dialog_search_cb (FindDialog *dialog, GaleonWindow *window)
{
	galeon_window_update_control (window, FindControl);
}

GaleonDialog *
galeon_window_get_find_dialog (GaleonWindow *window)
{			       
	GaleonDialog *dialog;
	GaleonEmbed *embed;

	if (window->priv->find_dialog)
	{
		return window->priv->find_dialog;
	}
	
	embed = galeon_window_get_active_embed (window);
	g_return_val_if_fail (GTK_IS_WINDOW(window), NULL);
	dialog = find_dialog_new_with_parent (GTK_WIDGET(window),
					      embed);

	g_signal_connect (dialog, "search", 
			  G_CALLBACK (find_dialog_search_cb),
			  window);
	
	window->priv->find_dialog = dialog;
	
	return dialog;
}

void
galeon_window_show_history (GaleonWindow *window)
{
	GaleonEmbed *embed;

	embed = galeon_window_get_active_embed (window);
	g_return_if_fail (embed != NULL);

	window->priv->history_dialog = history_dialog_new_with_parent 
						(GTK_WIDGET(window),
			 		     	 embed,
						 FALSE);
	g_object_add_weak_pointer 
                        (G_OBJECT(window->priv->history_dialog),
                         (gpointer *)&window->priv->history_dialog);

	galeon_dialog_show (window->priv->history_dialog);
}

static char *
galeon_window_gb_location_source_get_location (GbLocationSource *src)
{
	GaleonWindow *window;
	GaleonTab *tab;
	const gchar *location;

	window = GALEON_WINDOW (src);
	g_return_val_if_fail (window, NULL);

	tab = galeon_window_get_active_tab (window);
	g_return_val_if_fail (tab, NULL);

	location = galeon_tab_get_location (tab);
	if (!location) location = "";

	return g_strdup (location);
}

static char *
galeon_window_gb_location_source_get_title (GbLocationSource *src)
{
	GaleonWindow *window;
	GaleonTab *tab;
	const gchar *title;

	window = GALEON_WINDOW (src);
	g_return_val_if_fail (window, NULL);

	tab = galeon_window_get_active_tab (window);
	g_return_val_if_fail (tab, NULL);

	title = galeon_tab_get_title (tab);
	if (!title) 
	{
		return galeon_window_gb_location_source_get_location (src);
	}

	return g_strdup (title);
}

void
galeon_window_set_zoom (GaleonWindow *window, 
			gint zoom)
{
	GaleonEmbed *embed;

        g_return_if_fail (IS_GALEON_WINDOW (window));
        
	embed = galeon_window_get_active_embed (window);
        g_return_if_fail (embed != NULL);
	
        galeon_embed_zoom_set (embed, zoom, TRUE);
}

Toolbar *
galeon_window_get_toolbar (GaleonWindow *window)
{
	return window->priv->toolbar;
}

/**
 * Adds / updates an item to the window recent history menu 
 */
void
galeon_window_site_visited (GaleonWindow *window, const gchar *url, const gchar *title)
{
	g_return_if_fail (IS_GALEON_WINDOW (window));
	galeon_window_recent_history_visited (window->priv->recent_history, url, title);
}

static void
galeon_window_recent_history_menu_activated_cb (GaleonWindowRecentHistoryMenu *wrhm, 
						const char *url, const char *title, 
						GaleonWindow *w)
{
	galeon_window_load_url (w, url);
}

